@@ -80,7 +80,7 @@ int parse_args(int argc, char **argv,
 	       bool *hmac, bool *may_block,
 	       int *exit_code)
 {
-	char c;
+	int c;
 	bool hex_encoded = false;
 
 	while((c = getopt(argc, argv, optstring)) != -1) {
@@ -121,12 +121,12 @@ int parse_args(int argc, char **argv,
 	}
 
 	if (hex_encoded) {
-		static unsigned char decoded[64];
+		static unsigned char decoded[SHA1_MAX_BLOCK_SIZE];
 		int decoded_len;
 
 		int strl = strlen(argv[optind]);
 
-		if (strl >= sizeof(decoded) * 2) {
+		if (strl > sizeof(decoded) * 2) {
 			fprintf(stderr, "Hex-encoded challenge too long (max %i chars)\n",
 				sizeof(decoded) * 2);
 			return 0;
@@ -140,7 +140,7 @@ int parse_args(int argc, char **argv,
 		memset(decoded, 0, sizeof(decoded));
 
 		if (yubikey_hex_p(argv[optind])) {
-			yubikey_hex_decode((char *)decoded, argv[optind], strl);
+			yubikey_hex_decode((char *)decoded, argv[optind], sizeof(decoded));
 		} else {
 			fprintf(stderr, "Bad hex-encoded string '%s'\n", argv[optind]);
 			return 0;
@@ -173,7 +173,8 @@ int check_firmware(YK_KEY *yk, bool verbose)
 	}
 
 	if (ykds_version_major(st) < 2 ||
-	    ykds_version_minor(st) < 2) {
+	    (ykds_version_major(st) == 2
+	     && ykds_version_minor(st) < 2)) {
 		fprintf(stderr, "Challenge-response not supported before YubiKey 2.2.\n");
 		free(st);
 		return 0;
@@ -188,7 +189,7 @@ int challenge_response(YK_KEY *yk, int slot,
 		       bool hmac, bool may_block, bool verbose)
 {
 	unsigned char response[64];
-	unsigned char output_buf[sizeof(response) * 2];
+	unsigned char output_buf[(SHA1_MAX_BLOCK_SIZE * 2) + 1];
 	int yk_cmd;
 	unsigned int flags = 0;
 	unsigned int response_len = 0;
