@@ -1,6 +1,6 @@
 /* -*- mode:C; c-file-style: "bsd" -*- */
 /*
- * Copyright (c) 2008, 2009, Yubico AB
+ * Copyright (c) 2008-2011, Yubico AB
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -35,6 +35,10 @@
 #include <stdbool.h>
 #include <stdint.h>
 
+# ifdef __cplusplus
+extern "C" {
+# endif
+
 /*************************************************************************
  **
  ** N O T E :  For all functions that return a value, 0 and NULL indicates
@@ -142,4 +146,8 @@ const char *yk_usb_strerror();
 
 #define YK_CRC_OK_RESIDUAL	0xf0b8
 
+# ifdef __cplusplus
+}
+# endif
+
 #endif	/* __YKCORE_H_INCLUDED__ */
