@@ -309,6 +309,38 @@ static bool vcheck_v22_or_greater(const YKP_CONFIG *cfg)
 		cfg->yk_major_version > 2;
 }
 
+int ykp_set_oath_imf(YKP_CONFIG *cfg, unsigned long imf)
+{
+	if (!vcheck_v22_or_greater(cfg)) {
+		ykp_errno = YKP_EYUBIKEYVER;
+		return 0;
+	}
+	if (imf > 65535*16) {
+		ykp_errno = YKP_EINVAL;
+		return 0;
+	}
+	if (imf % 16 != 0) {
+		ykp_errno = YKP_EINVAL;
+		return 0;
+	}
+	/* IMF/16 is 16 bits stored big-endian in uid[4] */
+	imf /= 16;
+	cfg->ykcore_config.uid[4] = imf >> 8;
+	cfg->ykcore_config.uid[5] = imf;
+	return 1;
+}
+
+unsigned long ykp_get_oath_imf(const YKP_CONFIG *cfg)
+{
+	if (!vcheck_v22_or_greater(cfg)) {
+		return 0;
+	}
+
+	/* we can't do a simple cast due to alignment issues */
+	return ((cfg->ykcore_config.uid[4] << 8)
+		| cfg->ykcore_config.uid[5]) << 4;
+}
+
 #define def_set_charfield(fnname,fieldname,size,extra,vcheck)	\
 int ykp_set_ ## fnname(YKP_CONFIG *cfg, unsigned char *input, size_t len)	\
 {								\
@@ -431,6 +463,7 @@ const char str_fixed[] = "fixed";
 const char str_uid[] = "uid";
 const char str_key[] = "key";
 const char str_acc_code[] = "acc_code";
+const char str_oath_imf[] = "OATH IMF";
 
 const char str_flags_separator[] = "|";
 
@@ -520,7 +553,7 @@ int ykp_write_config(const YKP_CONFIG *cfg,
 		       userdata);
 		/* XXX print OATH-HOTP fixed differently based on oath-fixed-modhex etc. */
 		writer(str_modhex_prefix,
-		       strlen(str_key_value_separator),
+		       strlen(str_modhex_prefix),
 		       userdata);
 		yubikey_modhex_encode(buffer, (char *)cfg->ykcore_config.fixed, cfg->ykcore_config.fixedSize);
 		writer(buffer, strlen(buffer), userdata);
@@ -535,7 +568,7 @@ int ykp_write_config(const YKP_CONFIG *cfg,
 			writer("n/a", 3, userdata);
 		} else {
 			writer(str_hex_prefix,
-			       strlen(str_key_value_separator),
+			       strlen(str_hex_prefix),
 			       userdata);
 			yubikey_hex_encode(buffer, (char *)cfg->ykcore_config.uid, UID_SIZE);
 			writer(buffer, strlen(buffer), userdata);
@@ -548,7 +581,7 @@ int ykp_write_config(const YKP_CONFIG *cfg,
 		       strlen(str_key_value_separator),
 		       userdata);
 		writer(str_hex_prefix,
-		       strlen(str_key_value_separator),
+		       strlen(str_hex_prefix),
 		       userdata);
 		yubikey_hex_encode(buffer, (char *)cfg->ykcore_config.key, KEY_SIZE);
 		if (key_bits_in_uid) {
@@ -563,12 +596,27 @@ int ykp_write_config(const YKP_CONFIG *cfg,
 		       strlen(str_key_value_separator),
 		       userdata);
 		writer(str_hex_prefix,
-		       strlen(str_key_value_separator),
+		       strlen(str_hex_prefix),
 		       userdata);
 		yubikey_hex_encode(buffer, (char *)cfg->ykcore_config.accCode, ACC_CODE_SIZE);
 		writer(buffer, strlen(buffer), userdata);
 		writer("\n", 1, userdata);
 
+		/* OATH IMF: */
+		if ((cfg->ykcore_config.tktFlags & TKTFLAG_OATH_HOTP) == TKTFLAG_OATH_HOTP &&
+		    vcheck_v22_or_greater(cfg)) {
+			writer(str_oath_imf, strlen(str_oath_imf), userdata);
+			writer(str_key_value_separator,
+				strlen(str_key_value_separator),
+				userdata);
+			writer(str_hex_prefix,
+				strlen(str_hex_prefix),
+				userdata);
+			sprintf(buffer, "%lx", ykp_get_oath_imf(cfg));
+			writer(buffer, strlen(buffer), userdata);
+			writer("\n", 1, userdata);
+		}
+
 		/* ticket_flags: */
 		buffer[0] = '\0';
 		for (p = ticket_flags_map; p->flag; p++) {
@@ -674,8 +722,13 @@ int * const _ykp_errno_location(void)
 
 	if (tsd_init == 0) {
 		if ((rc = YK_TSD_INIT(errno_key, free)) == 0) {
-			YK_TSD_SET(errno_key, calloc(1, sizeof(int)));
-			tsd_init = 1;
+			void *p = calloc(1, sizeof(int));
+			if (!p) {
+				tsd_init = -1;
+			} else {
+				YK_TSD_SET(errno_key, p);
+				tsd_init = 1;
+			}
 		} else {
 			tsd_init = -1;
 		}
@@ -693,6 +746,7 @@ static const char *errtext[] = {
 	"option not available for this Yubikey version",
 	"too old yubikey for this operation",
 	"invalid configuration number (this is a programming error)",
+	"invalid option/argument value",
 };
 const char *ykp_strerror(int errnum)
 {
