@@ -1,6 +1,6 @@
 /* -*- mode:C; c-file-style: "bsd" -*- */
 /*
- * Copyright (c) 2008, 2009, Yubico AB
+ * Copyright (c) 2008-2011  Yubico AB
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -35,6 +35,10 @@
 #include <stdbool.h>
 #include <ykstatus.h>
 
+# ifdef __cplusplus
+extern "C" {
+# endif
+
 typedef struct ykp_config_t YKP_CONFIG;
 
 /* This only works with Yubikey 1 unless it's fed with a YK_STATUS using
@@ -54,6 +58,8 @@ int ykp_HMAC_key_from_hex(YKP_CONFIG *cfg, const char *hexkey);
 int ykp_set_access_code(YKP_CONFIG *cfg, unsigned char *access_code, size_t len);
 int ykp_set_fixed(YKP_CONFIG *cfg, unsigned char *fixed, size_t len);
 int ykp_set_uid(YKP_CONFIG *cfg, unsigned char *uid, size_t len);
+int ykp_set_oath_imf(YKP_CONFIG *cfg, unsigned long imf);
+unsigned long ykp_get_oath_imf(const YKP_CONFIG *cfg);
 
 int ykp_set_tktflag_TAB_FIRST(YKP_CONFIG *cfg, bool state);
 int ykp_set_tktflag_APPEND_TAB1(YKP_CONFIG *cfg, bool state);
@@ -109,5 +115,10 @@ const char *ykp_strerror(int errnum);
 #define YKP_EYUBIKEYVER	0x03
 #define YKP_EOLDYUBIKEY	0x04
 #define YKP_EINVCONFNUM	0x05
+#define YKP_EINVAL	0x06
 
-#endif	// __YKPERS_H_INCLUDED__
+# ifdef __cplusplus
+}
+# endif
+
+#endif	/* __YKPERS_H_INCLUDED__ */
